@@ -1,8 +1,8 @@
-'use strict'
+'use strict';
 
-var inflateSync = require('zlibjs').inflateSync;
+const inflateSync = require('zlibjs').inflateSync;
 
-exports.run = function(data, level) {
+exports.run = (data/*, level*/) => {
   // Compression levels not supported. Use unknown defaults always
   return inflateSync(data.deflateTyped);
-}
+};
